unit Win95pie;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Menus, ComAbout, DsgnIntf;

type
  TWin95PieChart = class(TCustomPanel)
  private
    { Private declarations }
    FValue, FMaximum, FDepth: integer;
    FUsedColor, FUsedShadowColor, FFreeColor, FFreeShadowColor, FLineColor: TColor;
    FAbout: string;
    procedure DrawPie;
    procedure SetValue(Value: integer);
    procedure SetMaximum(Maximum: integer);
    procedure SetDepth(Depth: integer);
    Procedure SetUsedColor(Color: TColor);
    Procedure SetUsedShadowColor(Color: TColor);
    Procedure SetFreeColor(Color: TColor);
    Procedure SetFreeShadowColor(Color: TColor);
    Procedure SetLineColor(Color: TColor);
  protected
    { Protected declarations }
    {$IFDEF WIN32}
    procedure ValidateRename(AComponent: TComponent; const CurName, NewName: string);  override;
    {$ENDIF}
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Paint; override;
  published
    { Published declarations }
    property Value: integer read FValue write SetValue;
    property Maximum: integer read FMaximum write SetMaximum;
    property Depth: integer read FDepth write SetDepth;
    property UsedColor: TColor read FUsedColor write SetUsedColor;
    property UsedShadowColor: TColor read FUsedShadowColor write SetUsedShadowColor;
    property FreeColor: TColor read FFreeColor write SetFreeColor;
    property FreeShadowColor: TColor read FFreeShadowColor write SetFreeShadowColor;
    property LineColor: TColor read FLineColor write SetLineColor;

    property Align;
    property BorderStyle;
    property Color;
    property HelpContext;
    property Hint;
    property ParentColor;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;

    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;

    property About: string read FAbout write FAbout;
  end;


      { AboutTypeProperty }

type
  TAboutTypeProperty = class(TPropertyEditor)
  private

  public
     function GetValue: string; override;
     procedure SetValue(const Value: string); override;
     procedure Edit; override;
     function GetAttributes: TPropertyAttributes; override;
  end;

var
   AboutWin95PieChart: TAboutComponent;


procedure Register;
procedure RunAboutBox;

implementation

procedure Register;
begin
  RegisterComponents('BealeARTS', [TWin95PieChart]);
  RegisterPropertyEditor(TypeInfo(string), TWin95PieChart, 'About', TAboutTypeProperty);
end;


{$IFDEF WIN32}
procedure TWin95PieChart.ValidateRename(AComponent: TComponent;
  const CurName, NewName: string);
begin
  if (AComponent <> nil) and (CompareText(CurName, NewName) <> 0) and
    (FindComponent(NewName) <> nil) then
    raise EComponentError.CreateFmt('A component named %s already exists.', [NewName]);
end;
{$ENDIF}


constructor TWin95PieChart.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  Width := 125 + 10;
  Height := 65 + 10;
  BevelOuter := bvNone;

  FValue := 80;
  FMaximum := 100;
  FDepth := 10;
  FUsedColor := clBlue;
  FUsedShadowColor := clNavy;
  FFreeColor := clFuchsia;
  FFreeShadowColor := clPurple;
  FLineColor := clBlack;

end;



procedure TWin95PieChart.DrawPie;
var
   X1, Y1, X2, Y2, Angle: integer;
   DegreeCount: Real;
begin

Angle := Round((FValue/FMaximum)*360);

DegreeCount := (-Angle+270);
X1 := Trunc( 0.5 * Width * SIN(((DegreeCount*PI) / 180)) );
Y1 := Trunc( 0.5 * (Height-FDepth) * COS(((DegreeCount*PI) / 180)) );
X1 := X1 + Trunc(Width/2);
Y1 := Y1 + Trunc((Height-FDepth)/2);

X2 := 0;
Y2 := Trunc((Height-FDepth)/2);

Canvas.Brush.color := Color;
Canvas.FillRect(Rect(0,0,Width,Height));

Canvas.Pen.Color := FLineColor;
Canvas.Brush.Color := FreeShadowColor;
Canvas.Pie(0,FDepth+0,Width,Height,0,Trunc((Height+FDepth)/2),Width,Trunc((Height+FDepth)/2));

Canvas.Pen.Color := FreeShadowColor;
Canvas.MoveTo(X2+1,Y2+FDepth);
Canvas.LineTo(Width-2,Y2+FDepth);
Canvas.Pen.Color := FLineColor;

Canvas.MoveTo(X2,Y2);
Canvas.LineTo(X2,Y2+FDepth);

Canvas.MoveTo(Width-1,Y2);
Canvas.LineTo(Width-1,Y2+FDepth);

if Y1 > Trunc((Height-FDepth)/2) then
begin
Canvas.MoveTo(X1,Y1);
Canvas.LineTo(X1,Y1+FDepth);
end;

Canvas.Brush.Color := FUsedColor;
Canvas.Pie(0,0,Width,Height-FDepth,X1,Y1,X2,Y2);

Canvas.Brush.Color := FreeColor;
Canvas.Pie(0,0,Width,Height-FDepth,X2,Y2,X1,y1);

if Y1 > Trunc((Height-FDepth)/2) then
begin
Canvas.Brush.Color := FreeShadowColor;
Canvas.FloodFill(X1-1 ,Y1+Trunc(FDepth/2),FLineColor,fsBorder);
Canvas.Brush.Color := UsedShadowColor;
Canvas.FloodFill(X1+1 ,Y1+Trunc(FDepth/2),FLineColor,fsBorder);
end
else
begin
Canvas.Brush.Color := FreeShadowColor;
Canvas.FloodFill(Trunc(Width/2),Height-Trunc(FDepth/2),FLineColor,fsBorder);
end;


end;



procedure TWin95PieChart.Paint;
begin
Caption := '';
   inherited Paint;
DrawPie;
end;



procedure TWin95PieChart.SetValue(Value: integer);
begin
if Value < 0 then exit;
FValue := Value;
DrawPie;
end;


procedure TWin95PieChart.SetMaximum(Maximum: integer);
begin
if Maximum < 1 then exit;
FMaximum := Maximum;
DrawPie;
end;


procedure TWin95PieChart.SetDepth(Depth: integer);
begin
if Depth < 0 then exit;
FDepth := Depth;
DrawPie;
end;


Procedure TWin95PieChart.SetUsedColor(Color: TColor);
begin
FUsedColor := Color;
DrawPie;
end;


Procedure TWin95PieChart.SetUsedShadowColor(Color: TColor);
begin
FUsedShadowColor := Color;
DrawPie;
end;


Procedure TWin95PieChart.SetFreeColor(Color: TColor);
begin
FFreeColor := Color;
DrawPie;
end;


Procedure TWin95PieChart.SetFreeShadowColor(Color: TColor);
begin
FFreeShadowColor := Color;
DrawPie;
end;


Procedure TWin95PieChart.SetLineColor(Color: TColor);
begin
FLineColor := Color;
DrawPie;
end;



procedure RunAboutBox;
begin
 { Create dialog in memory }
     AboutWin95PieChart := TAboutComponent.Create(Application);

     { Set dialog strings }
     AboutWin95PieChart.ProductName.Caption := 'Win95PieChart';
     AboutWin95PieChart.Caption := 'About Win95PieChart';
     AboutWin95PieChart.Copyright.Caption := 'Copyright 1998, David Beale';

AboutWin95PieChart.Memo.Lines.Add('Properties');
AboutWin95PieChart.Memo.Lines.Add('_______________________________________________');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('About - String');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Displays this about dialog. Performs no function when set from within your code.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Align - TAlign');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Determines how the control aligns within its container.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('BorderStyle - TBorderStyle');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Determines whether the control has a border.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Color - TColor');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Background color of the control');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Cursor - TCursor');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Cursor used by control.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Depth - Integer');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Depth of the pie chart.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('FreeColor - TColor');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Color of the free top part of the pie.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('FreeShadowColor - TColor');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Color of the free side part of the pie.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Height - Integer');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Vertical size of the control');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('HelpContext - THelpContext');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Context number for use in calling context-sensitive online Help.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Hint - String');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Control hint message');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Left - Integer');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Horizontal coordinate of the left edge of the control.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('LineColor - TColor');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Color of pie charts lines.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Maximum - Integer');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Largest value the pie chart can display');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Name - String');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Name of this component, ie PrinterAid1.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('ParentColor - Boolean');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Determines where the control looks for its color information.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('ParentShowHint - Boolean');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Determines where the control looks to find out if Help Hint should be show.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('PopupMenu - TPopupMenu');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Identifies the name of the control''s pop-up menu');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('ShowHint - Boolean');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Determines if the control should display a Help Hint');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('TabOrder - TTabOrder');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Indicates the position of the control in its parent''s tab order.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('TabStop - Boolean');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Determines if the user can tab to a control.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Tag - LongInt');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Components Tag.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Top - Integer');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Vertical coordinate of the top edge of the control.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('UsedColor - TColor');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Color of the used top part of the pie.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('UsedShadowColor - TColor');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Color of the used side part of the pie.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Value - Integer');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Value to be displayed on the pie chart');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Visible - Boolean');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Determines whether the control appears onscreen');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Width - Integer');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Horizontal size of the control.');

AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Events');
AboutWin95PieChart.Memo.Lines.Add('_______________________________________________');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('OnClick - TNotifyEvent');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Occurs when the user clicks the control.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('OnDblClick - TNotifyEvent');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Occurs when the user double-clicks the control.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('OnEnter - TNotifyEvent');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Occurs when a control becomes active.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('OnExit - TNotifyEvent');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Occurs when the input focus shifts away from the control to another.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('OnMouseDown - TNotifyEvent');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Occurs when the user presses a mouse button with the mouse pointer over the control.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('OnMouseMove - TNotifyEvent');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Occurs when the user moves the mouse pointer when the mouse pointer is over the control.');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('OnMouseUp - TNotifyEvent');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Occurs when the user releases a mouse button');
AboutWin95PieChart.Memo.Lines.Add('that was pressed with the mouse pointer over the control.');

AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Methods');
AboutWin95PieChart.Memo.Lines.Add('_______________________________________________');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Paint - Procedure');
AboutWin95PieChart.Memo.Lines.Add('');
AboutWin95PieChart.Memo.Lines.Add('Paints the pie chart. Usefull for forcing the painting of the pie chart.');



  { Set result to OK }
    AboutWin95PieChart.Icon := Application.Icon;
    AboutWin95PieChart.ShowModal;

AboutWin95PieChart.Free
end;



function TAboutTypeProperty.GetValue: string;
begin
Result := 'Win95PieChart';
end;

procedure TAboutTypeProperty.SetValue(const Value: string);
begin
RunAboutBox;
end;

procedure TAboutTypeProperty.Edit;
begin
RunAboutBox;
end;

function TAboutTypeProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

end.
